package codechicken.nei.asm;

import java.util.List;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.VarInsnNode;

import codechicken.core.asm.ASMHelper;
import codechicken.core.asm.ClassHeirachyManager;
import codechicken.core.asm.ClassOverrider;
import codechicken.core.asm.InstructionComparator;
import codechicken.core.asm.InstructionComparator.InsnListSection;
import codechicken.core.asm.ObfuscationMappings.ClassMapping;
import codechicken.core.asm.ObfuscationMappings.DescriptorMapping;
import static org.objectweb.asm.Opcodes.*;
import static org.objectweb.asm.ClassWriter.*;
import static codechicken.core.asm.InstructionComparator.*;

import cpw.mods.fml.relauncher.FMLRelauncher;
import cpw.mods.fml.relauncher.IClassTransformer;

public class NEITransformer implements IClassTransformer
{
    /**
     * Adds super.updateScreen() to non implementing GuiContainer subclasses
     */
    @SuppressWarnings("unchecked")
    public byte[] transformer001(String name, byte[] bytes)
    {
        ClassMapping classmap = new ClassMapping("net/minecraft/client/gui/inventory/GuiContainer");
        if(ClassHeirachyManager.classExtends(name, classmap.javaClass(), bytes))
        {
            ClassNode node = new ClassNode();
            ClassReader reader = new ClassReader(bytes);
            reader.accept(node, 0);

            DescriptorMapping methodmap = new DescriptorMapping("net/minecraft/client/gui/GuiScreen", "updateScreen", "()V");
            DescriptorMapping supermap = new DescriptorMapping(node.superName, methodmap);

            InsnList supercall = new InsnList();
            supercall.add(new VarInsnNode(ALOAD, 0));
            supercall.add(supermap.toInsn(INVOKESPECIAL));

            for(MethodNode methodnode : (List<MethodNode>) node.methods)
            {
                if(methodmap.matches(methodnode))
                {
                    InsnList importantNodeList = getImportantList(methodnode.instructions);
                    if(!insnListMatches(importantNodeList, supercall, 0))
                    {
                        methodnode.instructions.insertBefore(methodnode.instructions.getFirst(), supercall);
                        System.out.println("Inserted super call into " + name + "." + supermap.s_name);
                    }
                }
            }

            ClassWriter writer = new ClassWriter(COMPUTE_FRAMES | COMPUTE_MAXS);
            node.accept(writer);
            bytes = writer.toByteArray();
        }
        return bytes;
    }

    /**
     * Generates method for setting the placed position for the mob spawner item
     */
    public byte[] transformer002(String name, byte[] bytes)
    {
        ClassMapping classmap = new ClassMapping("net/minecraft/block/BlockMobSpawner");
        if(classmap.isClass(name))
        {
            ClassNode node = new ClassNode();
            ClassReader reader = new ClassReader(bytes);
            reader.accept(node, 0);

            DescriptorMapping methodmap = new DescriptorMapping("net/minecraft/block/Block", "onBlockPlacedBy", "(Lnet/minecraft/world/World;IIILnet/minecraft/entity/EntityLiving;)V");
            MethodNode methodnode = (MethodNode) node.visitMethod(ACC_PUBLIC, methodmap.s_name, methodmap.s_desc, null, null);

            methodnode.instructions.add(new VarInsnNode(ILOAD, 2));//param2
            methodnode.instructions.add(new FieldInsnNode(PUTSTATIC, "codechicken/nei/ItemMobSpawner", "placedX", "I"));
            methodnode.instructions.add(new VarInsnNode(ILOAD, 3));//param3
            methodnode.instructions.add(new FieldInsnNode(PUTSTATIC, "codechicken/nei/ItemMobSpawner", "placedY", "I"));
            methodnode.instructions.add(new VarInsnNode(ILOAD, 4));//param4
            methodnode.instructions.add(new FieldInsnNode(PUTSTATIC, "codechicken/nei/ItemMobSpawner", "placedZ", "I"));
            methodnode.instructions.add(new InsnNode(RETURN));

            ClassWriter writer = new ClassWriter(COMPUTE_FRAMES | COMPUTE_MAXS);
            node.accept(writer);
            bytes = writer.toByteArray();

            System.out.println("Generated BlockMobSpawner helper method.");
        }
        return bytes;
    }

    /**
     * Generates Slot.getBackgroundIconTexture for when forge is not installed.
     */
    @SuppressWarnings("unchecked")
    public byte[] transformer003(String name, byte[] bytes)
    {
        DescriptorMapping methodmap = new DescriptorMapping("net/minecraft/inventory/Slot", "getBackgroundIconTexture", "()Ljava/lang/String;");
        if(methodmap.isClass(name))
        {
            ClassNode node = new ClassNode();
            ClassReader reader = new ClassReader(bytes);
            reader.accept(node, 0);

            boolean declared = false;
            for(MethodNode method : (List<MethodNode>) node.methods)
            {
                if(methodmap.matches(method))
                {
                    declared = true;
                    break;
                }
            }

            if(!declared)
            {
                ClassWriter cw = new ClassWriter(COMPUTE_FRAMES);
                node.accept(cw);

                MethodVisitor mv = cw.visitMethod(ACC_PUBLIC, "getBackgroundIconTexture", "()Ljava/lang/String;", null, null);
                mv.visitCode();
                mv.visitLdcInsn("/gui/items.png");
                mv.visitInsn(ARETURN);
                mv.visitMaxs(1, 0);
                mv.visitEnd();

                bytes = cw.toByteArray();

                System.out.println("Generated default "+methodmap.s_owner+".getBackgroundIconTexture().");
            }
        }
        return bytes;
    }

    @Override
    public byte[] transform(String name, byte[] bytes)
    {
        try
        {
            if(FMLRelauncher.side().equals("CLIENT"))
            {
                bytes = transformer001(name, bytes);
                bytes = transformer002(name, bytes);
                bytes = transformer003(name, bytes);
                bytes = transformer004(name, bytes);
                bytes = transformer005(name, bytes);
                bytes = ClassOverrider.overrideBytes(name, bytes, new ClassMapping("net/minecraft/client/gui/inventory/GuiContainer"), NEICorePlugin.location);
            }
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }

        return bytes;
    }

    @SuppressWarnings("unchecked")
    private byte[] transformer005(String name, byte[] bytes)
    {
        ClassNode cnode = ASMHelper.createClassNode(bytes);
        boolean cont1 = false;
        boolean cont2 = false;
        for(String iclass : (List<String>)cnode.interfaces)
        {
            if(iclass.equals("codechicken/nei/api/INEIGuiHandler"))
                cont1 = true;
            if(iclass.equals("codechicken/nei/forge/IContainerInputHandler"))
                cont2 = true;
        }
        
        if(cont1)
        {
            DescriptorMapping mdesc = new DescriptorMapping("codechicken/nei/api/INEIGuiHandler", "handleDragNDrop", "(Lnet/minecraft/client/gui/inventory/GuiContainer;IILnet/minecraft/item/ItemStack;I)Z");
            MethodNode method = ASMHelper.findMethod(mdesc, cnode);
            if(method == null)
            {
                method = (MethodNode) cnode.visitMethod(ACC_PUBLIC, mdesc.s_name, mdesc.s_desc, null, null);
                method.visitInsn(ICONST_0);
                method.visitInsn(IRETURN);
            }
        }
        if(cont2)
        {
            DescriptorMapping mdesc = new DescriptorMapping("codechicken/nei/forge/IContainerInputHandler", "onMouseDragged", "(Lnet/minecraft/client/gui/inventory/GuiContainer;IIIJ)V");
            MethodNode method = ASMHelper.findMethod(mdesc, cnode);
            if(method == null)
            {
                method = (MethodNode) cnode.visitMethod(ACC_PUBLIC, mdesc.s_name, mdesc.s_desc, null, null);
                method.visitInsn(RETURN);
            }
        }
        
        if(cont1 || cont2)
            bytes = ASMHelper.createBytes(cnode, 3);
        return bytes;
    }

    private byte[] transformer004(String name, byte[] bytes)
    {
        DescriptorMapping mdesc = new DescriptorMapping("net/minecraft/client/gui/GuiScreen", "handleMouseInput", "()V");
        if(mdesc.isClass(name))
        {
            ClassNode cnode = ASMHelper.createClassNode(bytes);
            MethodNode mnode = ASMHelper.findMethod(mdesc, cnode);
            
            InsnList needle = new InsnList();
            needle.add(new VarInsnNode(ALOAD, 0));
            needle.add(new FieldInsnNode(GETFIELD, "net/minecraft/client/gui/GuiScreen", "mc", "Lnet/minecraft/client/Minecraft;"));
            needle.add(new FieldInsnNode(GETFIELD, "net/minecraft/client/Minecraft", "gameSettings", "Lnet/minecraft/client/settings/GameSettings;"));
            needle.add(new FieldInsnNode(GETFIELD, "net/minecraft/client/settings/GameSettings", "touchscreen", "Z"));
            needle.add(new JumpInsnNode(IFEQ, new LabelNode()));
            needle.add(new VarInsnNode(ALOAD, 0));
            needle.add(new FieldInsnNode(GETFIELD, "net/minecraft/client/gui/GuiScreen", "field_85042_b", "I"));
            
            List<InsnListSection> list = InstructionComparator.insnListFindL(mnode.instructions, needle);
            InsnListSection section = list.get(0);
            ASMHelper.removeSection(mnode.instructions, section.first, section.last.getPrevious().getPrevious());
            
            bytes = ASMHelper.createBytes(cnode, 3);
        }
        
        return bytes;
    }
}
